@@ -4,6 +4,14 @@ import archinstall
 
 is_top_level_profile = False
 
+# Note: GDM should be part of the gnome group, but adding it here for clarity
+__packages__ = [
+	"gnome",
+	"gnome-tweaks",
+	"gdm",
+]
+
+
 def _prep_function(*args, **kwargs):
 	"""
 	Magic function called by the importing installer
@@ -21,17 +29,17 @@ def _prep_function(*args, **kwargs):
 		else:
 			print('Deprecated (??): xorg profile has no _prep_function() anymore')
 
+
 # Ensures that this code only gets executed if executed
 # through importlib.util.spec_from_file_location("gnome", "/somewhere/gnome.py")
 # or through conventional import gnome
 if __name__ == 'gnome':
 	# Install dependency profiles
-	installation.install_profile('xorg')
+	archinstall.storage['installation_session'].install_profile('xorg')
 
-	# Install the application gnome from the template under /applications/
-	gnome = archinstall.Application(installation, 'gnome')
-	gnome.install()
+	# Install the GNOME packages
+	archinstall.storage['installation_session'].add_additional_packages(__packages__)
 
-	installation.enable_service('gdm') # Gnome Display Manager
-	# We could also start it via xinitrc since we do have Xorg,
-	# but for gnome that's deprecated and wayland is preferred.
+	archinstall.storage['installation_session'].enable_service('gdm')  # Gnome Display Manager
+# We could also start it via xinitrc since we do have Xorg,
+# but for gnome that's deprecated and wayland is preferred.
