@@ -1,10 +1,24 @@
-
 # A desktop environment using "LXQt"
 
 import archinstall
 
 is_top_level_profile = False
 
+# NOTE: SDDM is the only officially supported greeter for LXQt, so unlike other DEs, lightdm is not used here.
+# LXQt works with lightdm, but since this is not supported, we will not default to this.
+# https://github.com/lxqt/lxqt/issues/795
+__packages__ = [
+	"lxqt",
+	"breeze-icons",
+	"oxygen-icons",
+	"xdg-utils",
+	"ttf-freefont",
+	"leafpad",
+	"slock",
+	"sddm",
+]
+
+
 def _prep_function(*args, **kwargs):
 	"""
 	Magic function called by the importing installer
@@ -21,15 +35,16 @@ def _prep_function(*args, **kwargs):
 		else:
 			print('Deprecated (??): xorg profile has no _prep_function() anymore')
 
+
 # Ensures that this code only gets executed if executed
 # through importlib.util.spec_from_file_location("lxqt", "/somewhere/lxqt.py")
 # or through conventional import lxqt
 if __name__ == 'lxqt':
 	# Install dependency profiles
-	installation.install_profile('xorg')
+	archinstall.storage['installation_session'].install_profile('xorg')
 
-	# Install the application xfce4 from the template under /applications/
-	xfce = archinstall.Application(installation, 'lxqt')
-	xfce.install()
+	# Install the LXQt packages
+	archinstall.storage['installation_session'].add_additional_packages(__packages__)
 
-	installation.enable_service('sddm') # SDDM Display Manager
+	# Enable autostart of LXQt for all users
+	archinstall.storage['installation_session'].enable_service('sddm')
