@@ -1,9 +1,10 @@
 # Used to do a minimal install
 
-import archinstall, os
-
 is_top_level_profile = True
 
+__description__ = 'A very basic installation that allows you to customize Arch Linux as you see fit.'
+
+
 def _prep_function(*args, **kwargs):
 	"""
 	Magic function called by the importing installer
@@ -11,7 +12,8 @@ def _prep_function(*args, **kwargs):
 	we don't need to do anything special here, but it
 	needs to exist and return True.
 	"""
-	return True # Do nothing and just return True
+	return True  # Do nothing and just return True
+
 
 if __name__ == 'minimal':
 	"""
